if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[d%.][d%.][O%.]ns\\PitBull\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 39838 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Threat = PitBull:NewModule("Threat", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Threat
PitBull:SetModuleDefaultState(self, false)
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-06-14 04:02:50 -0400 (Thu, 14 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
self.desc = "Show a threat bar on the unit frame."

-- localization placeholder
local L = PitBull:L("PitBull-Threat")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local Threat

PitBull_Threat:RegisterPitBullChildFrames('threatBar')

function PitBull_Threat:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Threat")
	PitBull:RegisterDefaults("Threat", "profile", {
		['**'] = {
			ignore = false,
			deficit = false,
			reverse = false,
		},
		colors = {.75, 0, 0},
		bgColors = {.25, 0, 0},
	})
end

function PitBull_Threat:OnEnable(first)
	if not AceLibrary:HasInstance("Threat-1.0") then
		error("PitBull_Threat requires the library Threat-1.0 to be available.")
	end
	Threat = AceLibrary("Threat-1.0")
	self:RegisterEvent("Threat_ThreatUpdated")
	self:RegisterEvent("Threat_Activate")
	self:RegisterEvent("Threat_Deactivate")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("UNIT_PET", "PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PLAYER_PET_CHANGED", "PARTY_MEMBERS_CHANGED")
end

local playerRaidID
local petRaidID
function PitBull_Threat:PARTY_MEMBERS_CHANGED()
	local raid = GetNumRaidMembers()
	playerRaidID = nil
	petRaidID = nil
	if raid > 0 then
		for i = 1, raid do
			if UnitIsUnit("player", "raid" .. i) then
				playerRaidID = "raid" .. i
				break
			end
		end
		if UnitExists("pet") then
			for i = 1, raid do
				if UnitIsUnit("pet", "raidpet" .. i) then
					petRaidID = "raidpet" .. i
					break
				end
			end
		end
	end
end

local targetName
function PitBull_Threat:PLAYER_TARGET_CHANGED()
	targetName = UnitName("target")
	
	for unit, frame in PitBull:IterateUnitFrames() do
		self:UpdateThreat(unit, frame)
	end
end

function PitBull_Threat:Threat_ThreatUpdated(name, unit, hash, value)
	if not targetName then
		return
	end
	--self.Print("Threat Updated", name, unit, hash, value)
	if Threat:UnitIsUnit(targetName, hash) then
		self:UpdateThreat(unit)
	end
end

function PitBull_Threat:Threat_Activate()
	for unit, frame in PitBull:IterateUnitFrames() do
		self:OnPopulateUnitFrame(unit, frame)
		PitBull:UpdateLayout(frame)
	end
end

function PitBull_Threat:Threat_Deactivate()
	for unit, frame in PitBull:IterateUnitFrames() do
		self:OnClearUnitFrame(unit, frame)
		PitBull:UpdateLayout(frame)
	end
end

function PitBull_Threat:UpdateThreat(unit, frame)
	if not frame then
		for frame in PitBull:IterateUnitFramesForUnit(unit) do
			self:UpdateThreat(unit, frame)
		end
		if unit == "player" then
			if playerRaidID then
				self:UpdateThreat(playerRaidID)
			end
		elseif unit == "pet" then
			if petRaidID then
				self:UpdateThreat(petRaidID)
			end
		end
		return
	end
	if not frame.threatBar then
		return
	end
	
	local current, max
	if targetName then
		current, max = Threat:GetThreat(UnitName(unit), targetName), Threat:GetMaxThreatOnTarget(targetName)
	end
	if not max or max == 0 then
		current = 0
		max = 1
	end
	if self.db.profile[unit].deficit ~= self.db.profile[unit].reverse then
		frame.threatBar:SetValue(math.max(max - current, 0) / max)
	else
		frame.threatBar:SetValue(current / max)
	end
	local r, g, b = unpack(self.db.profile.colors)
	local br, bg, bb = unpack(self.db.profile.bgColors)
	if self.db.profile[unit].reverse then
		frame.threatBar:SetStatusBarColor(br, bg, bb, 1)
		frame.threatBar.bg:SetVertexColor(r, g, b, 1)
	else
		frame.threatBar:SetStatusBarColor(r, g, b, 1)
		frame.threatBar.bg:SetVertexColor(br, bg, bb, 1)
	end	
end

function PitBull_Threat:OnUpdateFrame(unit, frame)
	if not unit:find("target$") then
		self:UpdateThreat(unit, frame)
	end
end

function PitBull_Threat:OnPopulateUnitFrame(unit, frame)
	if frame.threatBar or self.db.profile[frame.group].ignore then
		return 
	end
	if unit:find("target$") then
		return
	end
	if not Threat:IsActive() then
		return
	end
	local threatBar = newFrame("StatusBar", frame)
	frame.threatBar = threatBar
	threatBar:SetMinMaxValues(0, 1)
	threatBar:SetValue(0.5)
	threatBar:SetStatusBarTexture(PitBull:GetStatusBarTexture())
	threatBar:SetStatusBarColor(unpack(self.db.profile.colors), 1)
	
	local threatBarBG = newFrame("Texture", threatBar, "BACKGROUND")
	frame.threatBar.bg = threatBarBG
	threatBarBG:SetTexture(PitBull:GetStatusBarTexture())
	threatBarBG:SetVertexColor(unpack(self.db.profile.bgColors))
	threatBarBG:SetAllPoints(threatBar)
end

function PitBull_Threat:OnClearUnitFrame(unit, frame)
	if frame.threatBar then
		frame.threatBar.bg = delFrame(frame.threatBar.bg)
		frame.threatBar = delFrame(frame.threatBar)
	end
end

function PitBull_Threat:OnUpdateStatusBarTexture(texture)
	for frame in PitBull:IterateUnitFramesForUnit("target") do
		if frame.threatBar and frame.threatBar.bg then
			frame.threatBar:SetStatusBarTexture(texture)
			frame.threatBar.bg:SetTexture(texture)
		end
	end
end

function PitBull_Threat:OnUnknownLayout(unit, frame, name)
	frame.threatBar:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", frame:GetWidth()/2, 5)
	frame.threatBar:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, 5)
	frame.threatBar:SetHeight(10)
end

local function getDisabled(group)
	return self.db.profile[group].ignore
end
local function getEnabled(group)
	return not self.db.profile[group].ignore
end
local function setEnabled(group, value)
	self.db.profile[group].ignore = not value
	
	if not value then
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			if frame.threatBar then
				PitBull_Threat:OnClearUnitFrame(unit, frame)
				PitBull:UpdateLayout(frame)
			end
		end
	else
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			if not frame.threatBar then
				PitBull_Threat:OnPopulateUnitFrame(unit, frame)
				PitBull_Threat:UpdateThreat(unit, frame)
				PitBull:UpdateLayout(frame)
			end
		end
	end
end

local function getDeficit(group)
	return PitBull_Threat.db.profile[group].deficit
end
local function setDeficit(group, value)
	PitBull_Threat.db.profile[group].deficit = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Threat:UpdateThreat(unit, frame)
	end
end
local function getReverse(group)
	return PitBull_Threat.db.profile[group].reverse
end
local function setReverse(group, value)
	PitBull_Threat.db.profile[group].reverse = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Threat:UpdateThreat(unit, frame)
	end
end


local function GetColor(group)
	return unpack(PitBull_Threat.db.profile.colors)
end
local function SetColor(group, r, g, b)
	PitBull_Threat.db.profile.colors = {r, g, b}
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Threat:UpdateThreat(unit, frame)
	end
end

local function GetBGColor(group)
	return unpack(PitBull_Threat.db.profile.bgColors)
end
local function SetBGColor(group, r, g, b)
	PitBull_Threat.db.profile.bgColors = {r, g, b}
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Threat:UpdateThreat(unit, frame)
	end
end

PitBull.options.args.global.args.colors.args.threatBar = {
	type = "group",
	name = L["Threat bar"],
	desc = L["Set the colors for the threat bar."],
	args = {
		bar = {
			type = "color",
			name = L["Color"],
			desc = L["Set the color for the threat bar."],
			get = GetColor,
			set = SetColor,
		},
		background = {
			type = "color",
			name = L["Background color"],
			desc = L["Set the background color for the threat bar."],
			get = GetBGColor,
			set = SetBGColor,
		},
	},
}

PitBull_Threat:RegisterPitBullOptionsMethod(function(group)
	if not group:find("target$") then
		return {
			name = L["Threat bar"],
			desc = L["Options for the units threat bar."],
			type = "group",
			args = {
				ignore = {
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable the threat bar."],
					get = getEnabled,
					set = setEnabled,
					passValue = group,
					order = 1,
				},
				deficit = {
					type = "toggle",
					name = L["Show as deficit"],
					desc = L["Show threat deficit, draining the bar when you gain threat."],
					get = getDeficit,
					set = setDeficit,
					passValue = group,
					disabled = getDisabled,
				},
				reverse = {
					type = "toggle",
					name = L["Reverse"],
					desc = L["Reverses the direction of the threat bar, filling it from right to left."],
					get = getReverse,
					set = setReverse,
					passValue = group,
					disabled = getDisabled,
				},
			}
		}
	end
end)
